/*
 * $Id: Ball.as,v 1.4 2007/09/06 12:35:41 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import org.cove.ape.Group;
  import org.cove.ape.CircleParticle;
  import abagames.util.Vector;

  /**
   * Ball that falls down in a room.
   */
  public class Ball extends ApeParticleActor {
    private var buffer:BitmapData;
    private var field:Field;
    private var player:Player;
    private var fragments:FragmentPool;
    private var particles:ParticlePool;
    private var smallEnergies:SmallEnergyPool;
    private var sound:SoundHandler;
    private var pvel:Vector;
    private var baseRadius:Number;
    private var radius:Number;
    private var hardness:Number;
    private var burstRadius:Number;
    private var rect:Rectangle;

    public function Ball(buffer:BitmapData, field:Field, player:Player,
                       fragments:FragmentPool, particles:ParticlePool, smallEnergies:SmallEnergyPool,
                       sound:SoundHandler, group:Group) {
      this.buffer = buffer;
      this.field = field;
      this.player = player;
      this.fragments = fragments;
      this.particles = particles;
      this.smallEnergies = smallEnergies;
      this.sound = sound;
      pvel = new Vector;
      _particle = new CircleParticle(0, 0, 6);
      setGroup(group);
      rect = new Rectangle(0, 0, 6, 6);
    }

    public function add(x:Number, y:Number, radius:Number, burstRadius:Number, hardness:Number):void {
      pvel.x = pvel.y = 0;
      addToGroup();
      _particle.visible = false;
      baseRadius = this.radius = radius;
      this.burstRadius = burstRadius;
      this.hardness = hardness;
      CircleParticle(_particle).radius = radius;
      _particle.elasticity = 0.3;
      _particle.mass = radius * 0.04;
      _particle.friction = 0.01;
      setPos(x, y);
    }

    public override function update():void {
      super.update();
      if (!field.isInY(_pos.y)) {
        remove();
        return;
      }
      if (Math.abs(_particle.velocity.x - pvel.x) + Math.abs(_particle.velocity.y - pvel.y) > 10) {
        radius *= (1 + 0.05 * (4 / hardness));
        CircleParticle(_particle).radius = radius;
        if (radius >= burstRadius) {
          destroy();
          return;
        }
      }
      pvel.x = _particle.velocity.x;
      pvel.y = _particle.velocity.y;
      var od:Number = Math.PI * 5 / Math.sqrt(hardness) / baseRadius;
      for (var d:Number = 0; d < Math.PI * 2; d += od) {
        rect.x = _pos.x + Math.sin(d) * radius - 3;
        rect.y = _pos.y + Math.cos(d) * radius - 3 + field.scrollY;
        buffer.fillRect(rect, 0xff0000);
      }
    }

    public function drawShape(p:Vector):void {
      var r:Number = 20;
      var od:Number = Math.PI * 0.125;
      for (var d:Number = 0; d < Math.PI * 2; d += od) {
        rect.x = p.x + Math.sin(d) * r - 3;
        rect.y = p.y + Math.cos(d) * r - 3;
        buffer.fillRect(rect, 0xff0000);
      }
    }

    public function destroy(withSe:Boolean = true):void {
      var i:int;
      var d:Number;
      for (i = 0; i < baseRadius * hardness * 0.4; i++) {
        var pt:Particle = Particle(particles.getInstanceForced());
        pt.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 5 + Math.random() * 15, 255, 0, 255);
      }
      if (withSe) {
        for (i = 0; i < baseRadius * hardness * 0.15; i++) {
          var f:Fragment = Fragment(fragments.getInstance());
          if (f == null)
            break;
          d = Math.random() * Math.PI * 2;
          f.add(_pos.x + Math.sin(d) * radius, _pos.y + Math.cos(d) * radius,
                d, 100 * (1 + Math.random()), 255, 0, 255);
        }
        for (i = 0; i < baseRadius * hardness * 0.025; i++) {
          var e:SmallEnergy = SmallEnergy(smallEnergies.getInstance());
          if (e == null)
            break;
          d = Math.random() * Math.PI * 2;
          e.add(_pos.x + Math.sin(d) * radius, _pos.y + Math.cos(d) * radius,
                d, 300 * (1 + Math.random()));
        }
        var sc:SoundChannel = sound.playSe(2);
        sound.setVolumeAndPan(sc, player.getSoundVolume(_pos), player.getSoundPan(_pos));
      }
      remove();
    }

    public override function remove():void {
      removeFromGroup();
      super.remove();
    }
  }
}
