/*
 * $Id: Cord.as,v 1.3 2007/09/06 12:35:41 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import org.cove.ape.Group;
  import org.cove.ape.RectangleParticle;
  import org.cove.ape.SpringConstraint;
  import abagames.util.Vector;

  /**
   * Cord connects ApeParticleActors.
   */
  public class Cord extends ApeParticleActor {
    private var buffer:BitmapData;
    private var field:Field;
    private var _actors:Array;
    private var length:Number;
    private var pointNum:int;
    private var offset:Vector;
    private var rect:Rectangle;
    private var blurRect:Rectangle;
    private var constraint:SpringConstraint;
    private var r:int;
    private var g:int;
    private var b:int;
    private var color:int;

    public function Cord(buffer:BitmapData, field:Field, group:Group):void {
      this.buffer = buffer;
      this.field = field;
      _actors = new Array;
      var i:int;
      for (i = 0; i < 2; i++)
        _actors.push(null);
      offset = new Vector;
      rect = new Rectangle(0, 0, 4, 4);
      blurRect = new Rectangle(0, 0, 4, 4);
      setGroup(group);
    }

    public function add(a1:ApeParticleActor, a2:ApeParticleActor, length:Number, springConstant:Number = 0.02, r:int = 0, g:int = 0, b:int = 0xff):void {
      _actors[0] = a1;
      _actors[1] = a2;
      this.length = length;
      pointNum = int(length / 4);
      constraint = new SpringConstraint(a1.particle, a2.particle, springConstant);
      constraint.restLength = length;
      group.addConstraint(constraint);
      constraint.visible = false;
      this.r = r;
      this.g = g;
      this.b = b;
      color = r * 0x10000 + g * 0x100 + b;
    }

    public override function update():void {
      draw();
    }

    private function draw():void {
      var vx:Number = _actors[1].pos.x - _actors[0].pos.x;
      var vy:Number = _actors[1].pos.y - _actors[0].pos.y;
      var cx:Number = _actors[0].pos.x;
      var cy:Number = _actors[0].pos.y;
      vx /= (pointNum + 1);
      vy /= (pointNum + 1);
      for (var i:int = 0; i < pointNum; i++) {
        cx += vx;
        cy += vy;
        rect.x = cx - 2;
        rect.y = cy - 2 + field.scrollY;
        buffer.fillRect(rect, color);
      }
    }

    public function drawShape(x1:Number, y1:Number, x2:Number, y2:Number):void {
      var vx:Number = x2 - x1;
      var vy:Number = y2 - y1;
      var cx:Number = x1;
      var cy:Number = y1;
      vx /= (pointNum + 1);
      vy /= (pointNum + 1);
      for (var i:int = 0; i < pointNum; i++) {
        cx += vx;
        cy += vy;
        rect.x = cx - 2;
        rect.y = cy - 2;
        buffer.fillRect(rect, color);
      }
    }

    public function drawShapeBlur(p1:Vector, p2:Vector, t:Number):void {
      var vx:Number = p2.x - p1.x;
      var vy:Number = p2.y - p1.y;
      var cx:Number = p1.x;
      var cy:Number = p1.y;
      vx /= (pointNum + 1);
      vy /= (pointNum + 1);
      var w:Number = 8 - t * 4;
      blurRect.width = blurRect.height = w;
      var sc:int = int(r * t) * 0x10000 + int(g * t) * 0x100 + int(b * t);
      for (var i:int = 0; i < pointNum; i++) {
        cx += vx;
        cy += vy;
        blurRect.x = cx - w / 2;
        blurRect.y = cy - w / 2 + field.scrollY;
        buffer.fillRect(blurRect, sc);
      }
    }

    public function removeConnected(a:ApeParticleActor):void {
      if (_actors[0] == a || _actors[1] == a)
        remove();
    }

    public override function remove():void {
      super.remove();
      group.removeConstraint(constraint);
    }

    public function get actors():Array {
      return _actors;
    }
  }
}
