/*
 * $Id: Mine.as,v 1.4 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.media.SoundChannel;
  import flash.media.SoundTransform;
  import org.cove.ape.Group;
  import org.cove.ape.CircleParticle;
  import abagames.util.Vector;

  /**
   * Mine that explodes when a player touches it.
   */
  public class Mine extends ApeParticleActor {
    private var buffer:BitmapData;
    private var field:Field;
    private var player:Player;
    private var fragments:FragmentPool;
    private var particles:ParticlePool;
    private var smallEnergies:SmallEnergyPool;
    private var sound:SoundHandler;
    private var pvel:Vector;
    private var cnt:int;
    private var damage:int;
    private var shapes:Array;

    public function Mine(buffer:BitmapData, field:Field, player:Player,
                       fragments:FragmentPool, particles:ParticlePool, smallEnergies:SmallEnergyPool,
                       sound:SoundHandler, group:Group) {
      this.buffer = buffer;
      this.field = field;
      this.player = player;
      this.fragments = fragments;
      this.particles = particles;
      this.smallEnergies = smallEnergies;
      this.sound = sound;
      pvel = new Vector;
      shapes = new Array;
      var shape:Shape;
      shape = new Shape(buffer, field);
      shape.addBox(0, 0, 16, 16, 150, 150, 150);
      shape.addBox(0, -14, 7, 7, 255, 255, 255);
      shape.addBox(-12, -6, 7, 7, 255, 255, 255);
      shape.addBox(12, -6, 7, 7, 255, 255, 255);
      shape.addBox(-12, 6, 7, 7, 255, 255, 255);
      shape.addBox(12, 6, 7, 7, 255, 255, 255);
      shape.addBox(0, 14, 7, 7, 255, 255, 255);
      shape.addBox(0, 0, 7, 7, 255, 255, 255);
      shapes.push(shape);
      shape = new Shape(buffer, field);
      shape.addBox(0, 0, 16, 16, 150, 150, 150);
      shape.addBox(0, -14, 7, 7, 255, 55, 55);
      shape.addBox(-12, -6, 7, 7, 255, 55, 55);
      shape.addBox(12, -6, 7, 7, 255, 55, 55);
      shape.addBox(-12, 6, 7, 7, 255, 55, 55);
      shape.addBox(12, 6, 7, 7, 255, 55, 55);
      shape.addBox(0, 14, 7, 7, 255, 55, 55);
      shape.addBox(0, 0, 7, 7, 255, 55, 55);
      shapes.push(shape);
      _particle = new CircleParticle(0, 0, 12);
      setGroup(group);
    }

    public function add(x:Number, y:Number):void {
      setPos(x, y);
      pvel.x = pvel.y = 0;
      cnt = 0;
      damage = 0;
      addToGroup();
      _particle.visible = false;
      _particle.mass = 10;
      _particle.friction = 0.033;
    }

    public override function update():void {
      super.update();
      if (!field.isInYUpper(_pos.y)) {
        remove();
        return;
      }
      if (Math.abs(_particle.velocity.x - pvel.x) + Math.abs(_particle.velocity.y - pvel.y) > 2) {
        damage++;
        if (damage >= 5) {
          destroy();
          return;
        }
      }
      pvel.x = _particle.velocity.x;
      pvel.y = _particle.velocity.y;
      if (player.hasCollision && _pos.dist(player.pos) < 24) {
        player.destroy();
        return;
      }
      cnt++;
      var pt:Particle = Particle(particles.getInstanceForced());
      pt.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 3, 255, 255, 150, 0);
      addForce(0, -Field.GRAVITY * (10 + Math.sin(cnt * 0.2) * 30) * 0.5);
      var sn:int = 0;
      if (damage > 0 && (cnt % ((5 - damage) * 2) == 0))
        sn = 1;
      shapes[sn].draw(_pos);
    }

    public function drawShape(p:Vector):void {
      shapes[0].drawWithoutScroll(p);
    }

    public function destroy(withSe:Boolean = true):void {
      var i:int;
      var d:Number;
      for (i = 0; i < 100; i++) {
        var pt:Particle = Particle(particles.getInstanceForced());
        pt.add(_pos.x, _pos.y, Math.random() * Math.PI * 2, 1 + Math.random() * 18, 255, 255, 150);
      }
      if (withSe) {
        for (i = 0; i < 15; i++) {
          var f:Fragment = Fragment(fragments.getInstance());
          if (f == null)
            break;
          d = Math.random() * Math.PI * 2;
          f.add(_pos.x + Math.sin(d) * 10, _pos.y + Math.cos(d) * 10,
                d, 200 * (1 + Math.random()), 255, 255, 150);
        }
        for (i = 0; i < 10; i++) {
          var e:SmallEnergy = SmallEnergy(smallEnergies.getInstance());
          if (e == null)
            break;
          d = Math.random() * Math.PI * 2;
          e.add(_pos.x + Math.sin(d) * 15, _pos.y + Math.cos(d) * 15,
                d, 300 * (1 + Math.random()));
        }
        var sc:SoundChannel = sound.playSe(3);
        sound.setVolumeAndPan(sc, player.getSoundVolume(_pos), player.getSoundPan(_pos));
      }
      remove();
    }

    public override function remove():void {
      removeFromGroup();
      super.remove();
    }
  }
}
