/*
 * $Id: Particle.as,v 1.2 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import flash.geom.Rectangle;
  import abagames.util.Vector;
  import abagames.util.SimpleActor;

  /**
   * Particle.
   */
  public class Particle extends SimpleActor {
    private var rect:Rectangle;
    private var buffer:BitmapData;
    private var field:Field;
    private var pos:Vector;
    private var vel:Vector;
    private var color:int;
    private var size:Vector;
    private var gravityRatio:Number;
    private var cnt:int;

    public function Particle(buffer:BitmapData, field:Field) {
      this.buffer = buffer;
      this.field = field;
      pos = new Vector;
      vel = new Vector;
      rect = new Rectangle(0, 0, 3, 3);
      size = new Vector(3, 3);
    }

    public function add(x:Number, y:Number, d:Number, s:Number, r:int, g:int, b:int, gr:Number = 1):void {
      addVel(x, y, Math.sin(d) * s, Math.cos(d) * s, r, g, b, gr);
    }

    public function addVel(x:Number, y:Number, vx:Number, vy:Number, r:int, g:int, b:int, gr:Number = 1):void {
      pos.x = x;
      pos.y = y;
      vel.x = vx;
      vel.y = vy;
      color = 0x10000 * r + 0x100 * g + b;
      gravityRatio = gr;
      cnt = 60 + Math.random() * 20;
    }

    public override function update():void {
      pos.x += vel.x;
      pos.y += vel.y;
      vel.x *= 0.95;
      vel.y *= 0.95;
      vel.y += Field.GRAVITY * 0.15 * gravityRatio;
      if ((pos.x <= field.upperLeftPos.x && vel.x < 0) ||
          (pos.x >= field.lowerRightPos.x && vel.x > 0))
        vel.x *= -0.8;
      if (!field.isInYScreen(pos.y)) {
        remove();
        return;
      }
      rect.x = pos.x - 1;
      rect.y = pos.y - 1 + field.scrollY;
      buffer.fillRect(rect, color);
      cnt--;
      if (vel.sizeRough < 1 || cnt < 0)
        remove();
    }
  }
}
