/*
 * $Id: Shape.as,v 1.2 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {
  import flash.display.BitmapData;
  import abagames.util.Vector;

  /**
   * Shape consists of boxes.
   */
  public class Shape {
    public static var distortion:Number = 1;
    private var buffer:BitmapData;
    private var field:Field;
    private var boxes:Array;

    public function Shape(buffer:BitmapData, field:Field) {
      this.buffer = buffer;
      this.field = field;
      boxes = new Array;
    }

    public function addBox(x:Number, y:Number, w:Number, h:Number,
                           r:int, g:int, b:int):void {
      var box:Box = new Box(buffer, field, x, y, w, h, r, g, b);
      boxes.push(box);
    }

    public function setColor(r:int, g:int, b:int):void {
      for each (var box:Box in boxes)
        box.setColor(r, g, b);
    }

    public function draw(p:Vector):void {
      for each (var box:Box in boxes)
        box.draw(p);
    }

    public function drawWithoutScroll(p:Vector):void {
      for each (var box:Box in boxes)
        box.drawWithoutScroll(p);
    }

    public function drawSized(p:Vector, size:Number):void {
      for each (var box:Box in boxes)
        box.drawSized(p, size);
    }

    public function drawBlur(p:Vector, s:Number):void {
      for each (var box:Box in boxes)
        box.drawBlur(p, s);
    }

    public function drawTilt(p:Vector, deg:Number):void {
      for each (var box:Box in boxes)
        box.drawTilt(p, deg);
    }

    public function drawDistorted(p:Vector):void {
      for each (var box:Box in boxes)
        box.drawDistorted(p);
    }

    public function drawDistortedWithoutScroll(p:Vector):void {
      for each (var box:Box in boxes)
        box.drawDistortedWithoutScroll(p);
    }
  }
}

import flash.display.BitmapData;
import flash.geom.Rectangle;
import abagames.util.Vector;
import abagames.lbr.Field;
import abagames.lbr.Shape;

class Box {
  private var buffer:BitmapData;
  private var field:Field;
  private var pos:Vector;
  private var size:Vector;
  private var depth:Number;
  private var rect:Rectangle;
  private var color:uint, depthColor:uint;
  private var tp:Vector;
  private var r:int;
  private var g:int;
  private var b:int;

  public function Box(buffer:BitmapData, field:Field,
                      x:Number, y:Number, w:Number, h:Number,
                      r:int, g:int, b:int) {
    this.buffer = buffer;
    this.field = field;
    pos = new Vector(x - w / 2, y - h / 2);
    size = new Vector(w, h);
    rect = new Rectangle(0, 0, w, h);
    setColor(r, g, b);
    tp = new Vector;
  }

  public function setColor(r:int, g:int, b:int):void {
    color = r * 0x10000 + g * 0x100 + b;
    this.r = r;
    this.g = g;
    this.b = b;
  }

  public function draw(p:Vector):void {
    rect.x = p.x + pos.x;
    rect.y = p.y + pos.y + field.scrollY;
    drawRect(p);
  }

  public function drawWithoutScroll(p:Vector):void {
    rect.x = p.x + pos.x;
    rect.y = p.y + pos.y;
    drawRect(p);
  }

  public function drawSized(p:Vector, size:Number):void {
    rect.x = p.x + pos.x * size;
    rect.y = p.y + pos.y * size + field.scrollY;
    drawRect(p);
  }

  public function drawBlur(p:Vector, t:Number):void {
    rect.x = p.x + pos.x - size.x * (2 - t - 1) / 2;
    rect.y = p.y + pos.y + field.scrollY - size.y * (2 - t - 1) / 2;
    rect.width = size.x * (2 - t);
    rect.height = size.y * (2 - t);
    var sc:int = int(r * t) * 0x10000 + int(g * t) * 0x100 + int(b * t);
    buffer.fillRect(rect, sc);
  }

  public function drawTilt(p:Vector, deg:Number):void {
    tp.x = Math.cos(deg) * pos.x - Math.sin(deg) * pos.y;
    tp.y = Math.sin(deg) * pos.x + Math.cos(deg) * pos.y;
    rect.x = p.x - tp.x;
    rect.y = p.y + tp.y + field.scrollY;
    drawRect(p);
  }

  public function drawDistorted(p:Vector):void {
    rect.x = p.x + pos.x;
    rect.y = p.y + pos.y + field.scrollY;
    rect.width = size.x * Shape.distortion;
    rect.height = size.y * Shape.distortion;
    drawRect(p);
  }

  public function drawDistortedWithoutScroll(p:Vector):void {
    rect.x = p.x + pos.x;
    rect.y = p.y + pos.y;
    rect.width = size.x * Shape.distortion;
    rect.height = size.y * Shape.distortion;
    drawRect(p);
  }

  private function drawRect(p:Vector):void {
    buffer.fillRect(rect, color);
  }
}
