/*
 * $Id: Stage.as,v 1.5 2007/09/06 12:35:42 kenta Exp $
 *
 * Copyright 2007 Kenta Cho. Some rights reserved.
 */
package abagames.lbr {

  /**
   * Handling an appearance pattern of balls, mines, energy items and floors.
   */
  public class Stage {
    private static const DEFAULT_CORD_LENGTH:Number = 16;
    private static const NEXT_BALL_APPEARE_CNT:int = 20;
    private var balls:BallPool;
    private var solidBalls:SolidBallPool;
    private var cords:CordPool;
    private var mines:MinePool;
    private var energies:EnergyPool;
    private var floors:FloorPool;
    private var field:Field;
    private var appearancePattern:int;
    private var addBallHeight:Number;
    private var addFloorHeight:Number;
    private var addMineHeight:Number;
    private var addEnergyHeight:Number;
    private var cnt:int;

    public function Stage(particles:ParticlePool, player:Player,
                        balls:BallPool, solidBalls:SolidBallPool, cords:CordPool,
                        mines:MinePool, energies:EnergyPool,
                        floors:FloorPool, field:Field) {
      this.balls = balls;
      this.solidBalls = solidBalls;
      this.cords = cords;
      this.mines = mines;
      this.energies = energies;
      this.floors = floors;
      this.field = field;
    }

    public function init():void {
      addBallHeight = 0;
      addMineHeight = 500;
      addFloorHeight = 100;
      addEnergyHeight = 50;
      cnt = 0;
    }

    public function update():void {
      cnt++;
      addBallHeight -= field.scrollVel;
      if (addBallHeight <= 0) {
        var bi:int = (60 + Math.random() * 10) / getDifficulty(900);
        if (bi < 10)
          bi = 10;
        addBallHeight += bi;
        addBall();
      }
      addFloorHeight -= field.scrollVel;
      if (addFloorHeight <= 0) {
        addFloorHeight += 150 + Math.random() * 400;
        var fx1:Number, fx2:Number;
        var r:Number = Math.random();
        if (r < 0.48) {
          fx1 = field.upperLeftPos.x + Floor.WIDTH * 0.4;
          fx2 = field.upperLeftPos.x + field.size.x * (0.33 + Math.random() * 0.33);
        } else if (r < 0.96) {
          fx1 = field.lowerRightPos.x - field.size.x * (0.33 + Math.random() * 0.33);
          fx2 = field.lowerRightPos.x - Floor.WIDTH * 0.4;
        } else {
          fx1 = field.upperLeftPos.x + Floor.WIDTH * 0.4;
          fx2 = field.lowerRightPos.x - Floor.WIDTH * 0.4;
        }
        addFloor(fx1, fx2, field.upperLeftPos.y);
      }
      addMineHeight -= field.scrollVel;
      if (addMineHeight <= 0) {
        var mi:int = (500 + Math.random() * 250) / getDifficulty(1200);
        if (mi < 20)
          mi = 20;
        addMineHeight += mi;
        addMine();
      }
      addEnergyHeight -= field.scrollVel;
      if (addEnergyHeight <= 0) {
        addEnergyHeight += 640;
        addEnegry();
      }
    }

    private function getDifficulty(cycle:int):Number {
      var cc:Number = Number(cnt % cycle) / cycle;
      var base:Number = int(cnt / cycle);
      return 1 + cc * (base + 1) * 2;
    }

    public function get energyDecreaseAmount():Number {
      return 7 + Math.sqrt(cnt) * 0.02;
    }

    private function addFloor(fx1:Number, fx2:Number, fy:Number):void {
      var f:Floor = Floor(floors.getInstance());
      if (f == null)
        return;
      var fx:Number = fx1;
      f.add(fx1, fy, true);
      var pf:Floor;
      var fn:int = int((fx2 - fx1) / 64) + 1;
      for (var i:int = 0; i < fn; i++) {
        pf = f;
        fx += (fx2 - fx1) / fn;
        f = Floor(floors.getInstance());
        if (f == null)
          break;
        f.add(fx, fy, (i == fn - 1));
        var c:Cord = Cord(cords.getInstance());
        if (c == null)
          break;
        c.add(pf, f, (fx2 - fx1) / fn * 0.5, 0.5);
      }
    }

    private function addBall():void {
      var bx:Number = field.upperLeftPos.x + field.size.x * (Math.random() * 0.8 + 0.1);
      var by:Number = field.upperLeftPos.y - field.size.y * 0.2;
      var r:Number = 20 + Math.random() * 30;
      var h:Number;
      if (Math.random() < 0.5)
        h = 1;
      else
        h = getDifficulty(1800) + Math.random();
      if (h > 6)
        h = 6;
      if (Math.random() < 0.5 - (1 / getDifficulty(2400))) {
        var sb:SolidBall = SolidBall(solidBalls.getInstance());
        if (sb != null) {
          sb.add(bx, by, r);
          return;
        }
      }
      var b:Ball = Ball(balls.getInstance());
      if (b == null)
        return;
      b.add(bx, by, r, r * 1.5, h);
    }

    private function addMine():void {
      var m:Mine = Mine(mines.getInstance());
      if (m != null) {
        var rn:Number = Math.random();
        var mx:Number = field.upperLeftPos.x;
        if (rn < 0.2)
          mx += field.size.x * 0.05;
        else if (rn > 0.8)
          mx += field.size.x * 0.95;
        else
          mx += field.size.x * (0.1 + Math.random() * 0.8);
        m.add(mx, field.upperLeftPos.y - field.size.y * 0.05);
      }
    }

    private function addEnegry():void {
      var e:Energy = Energy(energies.getInstance());
      if (e != null)
        e.add(field.upperLeftPos.x + field.size.x * (0.2 + Math.random() * 0.6), field.upperLeftPos.y - field.size.y * 0.05);
    }
  }
}
